/*
 * jMax
 * Copyright (C) 1994, 1995, 1998, 1999 by IRCAM-Centre Georges Pompidou, Paris, France.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * See file LICENSE for further informations on licensing terms.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Based on Max/ISPW by Miller Puckette.
 *
 * Authors: Maurizio De Cecco, Francois Dechelle, Enzo Maggi, Norbert Schnell.
 *
 * Author : Paul-Mathieu Brossier <Paul.Brossier@ircam.fr>
 */

#include <fts/fts.h>

#include <stdio.h> 
#include <stdlib.h>

#include <GL/glx.h>
typedef struct _pixmaprec_
{
  fts_object_t o;
  fts_symbol_t filename;
}pixmaprec_t;

typedef struct _BMPHeader_ {
  unsigned short type;
  unsigned int size;
  unsigned short res1;
  unsigned short res2;
  unsigned int offset;
}BMPHeader_t;
typedef struct _BMPInfo_ {
  unsigned int size;
  unsigned int width;
  unsigned int height;
  unsigned short planes;
  unsigned short bit_count;
  unsigned int comp;
  unsigned int sizeImage;
  unsigned int x_pels_per_meter;
  unsigned int y_pels_per_meter;
  unsigned int clr_used;
  unsigned int clr_important;
}BMPInfo_t;

typedef struct TGAheader {
  unsigned char idfield_len;
  unsigned char cmap_type;
  unsigned char image_type;
  unsigned char cmap_spec[5];
  unsigned char x_orig[2];
  unsigned char y_orig[2];
  unsigned char width[2];
  unsigned char height[2];
  unsigned char pixel_size;
  unsigned char image_desc;
} TGAHeader_t;

static void
gfx_tga_screenshot(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at){
  pixmaprec_t *this = (pixmaprec_t *)o;
  TGAHeader_t header;
  int w = 600;
  int h = 600;
  unsigned char *image = (unsigned char *) fts_malloc(sizeof(char) * w * h * 3);
  unsigned char temp;
  int i;
  FILE *fp;

  glReadPixels(0, 0, w, h, GL_BGR, GL_UNSIGNED_BYTE, image);

  fp = fopen(fts_symbol_name(this->filename), "wb");
  if (!fp) {
    post("Unable to open %s for writing\n", fts_symbol_name(this->filename));
    return;
  } else {
      /* Construct header */
      header.idfield_len = 0;
      header.cmap_type = 0;
      header.image_type = 2;
      for (i = 0; i < 5; i++) {
	header.cmap_spec[i] = 0;
      }
      for (i = 0; i < 2; i++) {
	header.x_orig[i] = 0;
	header.y_orig[i] = 0;
      }
      /* Lo bits */
      header.width[0] = w & 0xFF;
      /* Hi bits */
      header.width[1] = (w >> 8) & 0xFF;
      header.height[0] = h & 0xFF;
      header.height[1] = (h >> 8) & 0xFF;
      header.pixel_size = 24;
      header.image_desc = 0;
      /* Output header */
      fwrite(&header, sizeof(header), 1, fp);
      
      /* Output image */
      fwrite(image, sizeof(unsigned char), w*h*3, fp);
  }
  if (fp) {
    fclose(fp);
  }
  if(image)
    fts_free(image);
  return;
}

static void
gfx_bmp_screenshot(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at){
  pixmaprec_t *this = (pixmaprec_t *)o;
  BMPHeader_t header;
  BMPInfo_t info;
  int w = 600;
  int h = 600;
  unsigned char *image = (unsigned char *) fts_malloc(w * h * 3);
  unsigned char temp;
  int i;
  FILE *fp;

  glReadPixels(0, 0, w, h, GL_BGR, GL_UNSIGNED_BYTE, image);

  fp = fopen(fts_symbol_name(this->filename), "wb");
  if (!fp) {
    post("Unable to open %s for writing\n", fts_symbol_name(this->filename));
    return;
  } else {
    /* Header */
    header.type = 'B' | 'M' << 8;
    header.size = sizeof(header) + sizeof(info) + w * h * 3;
    header.res1 = header.res2 = 0;
    header.offset = sizeof(header) + sizeof(info);
    info.size = sizeof(info);
    info.width = w;
    info.height = h;
    info.planes = 1;
    info.bit_count = 24;
    info.comp = 0;
    info.sizeImage = w * h * 3;
    info.x_pels_per_meter = info.y_pels_per_meter = 0;
    info.clr_used = 0;
    info.clr_important = 0;
    fwrite(&header, sizeof(header), 1, fp);
    fwrite(&info, sizeof(info), 1, fp);
    
    fwrite(image, sizeof(unsigned char), h*w*3, fp);
  }
  if (fp) {
    fclose(fp);
  }
  if(image)
    fts_free(image);
  return;
}

static void
gfx_recbmp_init(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  pixmaprec_t *this = (pixmaprec_t *)o;
  ac--;
  at++; 

  if( ac >=0 && fts_is_symbol(at))
    {
      this->filename = fts_get_symbol(at); 
    }
}

static void
gfx_recbmp_delete(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{ 
  pixmaprec_t *this = (pixmaprec_t *)o;
  //  fts_object_release(this->o);
}

static fts_status_t
gfx_recbmp_instantiate(fts_class_t *cl, int ac, const fts_atom_t *at)
{

  fts_class_init(cl, sizeof(pixmaprec_t), 1, 0, 0);
  
  fts_method_define_varargs(cl, fts_SystemInlet, fts_s_init, gfx_recbmp_init);
  fts_method_define_varargs(cl, fts_SystemInlet, fts_s_delete, gfx_recbmp_delete);

  /*inlet 0*/
  fts_method_define_varargs(cl, 0, fts_s_bang, gfx_tga_screenshot);

  return fts_Success;

}

void
gfx_recbmp_config(void)
{
  fts_class_install(fts_new_symbol("Gwritetga"), gfx_recbmp_instantiate);
}
