/*
 * jMax
 * Copyright (C) 1994, 1995, 1998, 1999 by IRCAM-Centre Georges Pompidou, Paris, France.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * See file LICENSE for further informations on licensing terms.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Based on Max/ISPW by Miller Puckette.
 *
 * Authors: Maurizio De Cecco, Francois Dechelle, Enzo Maggi, Norbert Schnell.
 *
 * Author : Paul-Mathieu Brossier <Paul.Brossier@ircam.fr>
 */

#include <fts/fts.h>

   

#include <../packages/video/c/src/vectors/vectors.h>

#define DEFAULT_VIDWIN_TITLE "Video"
#define DEFAULT_VIDWIN_WIDTH 400
#define DEFAULT_VIDWIN_HEIGTH 300
#define DEFAULT_VIDWIN_XPOS 10
#define DEFAULT_VIDWIN_YPOS 10

//static fts_symbol_t rgba = 0;
static fts_symbol_t index = 0;
//static fts_symbol_t singlebuf = 0;
static fts_symbol_t doublebuf = 0;
static fts_symbol_t depthbuf = 0;
static fts_symbol_t accumbuf = 0;
static fts_symbol_t stencilbuf = 0;
static fts_symbol_t noborders = 0;

typedef struct _glutscreen_
{
  fts_object_t o;
  unsigned int mode;
  int position[2];
  int size[2];
  int fullscreen;
  int bgcolor[4];
  char *title;
} glutscreen_t;

GLuint currentlist;

static void
glutscreen_displaymode(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  glutscreen_t *this = (glutscreen_t *)o;  
  if(this->mode)
    glutInitDisplayMode(this->mode);
  else
    glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
}

//won't work with all window managers.
static void
glutscreen_window_position(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  glutscreen_t *this = (glutscreen_t *)o;
  if (this->fullscreen) 
    {
      this->position[0] = 0;
      this->position[1] = 0;
    }
  if ( this->position[0]  && this->position[1] )
    glutInitWindowPosition( this->position[0], this->position[1] );
  else
    glutInitWindowPosition( 10 , 10 );
}

static void
glutscreen_window_size(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  glutscreen_t *this = (glutscreen_t *)o;  
  if ( this->size[0] && this->size[1] )
    glutInitWindowSize( this->size[0] , this->size[1] );
  else
    glutInitWindowSize( 400, 400);
}
 
//the window may now be displayed
static void
glutscreen_window_title(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  glutscreen_t *this = (glutscreen_t *)o;
  if (this->title)
    glutCreateWindow(this->title);  
  else
    glutCreateWindow(DEFAULT_VIDWIN_TITLE);  
}

static void
glutscreen_shade_model(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  glutscreen_t *this = (glutscreen_t *)o;
  glShadeModel (GL_FLAT);
}

static void
glutscreen_background_color(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  glutscreen_t *this = (glutscreen_t *)o;
  glClearColor(0.0, 0.0, 0.0, 1.0);
}

void 
display(void)
{
   GLuint i;
   glClear (GL_COLOR_BUFFER_BIT);
   glCallList (currentlist);
   glFlush ();
}

void 
nothing(void)
{
  return;
}

void 
reshape(int w, int h)
{
   glViewport(0, 0, w, h);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   if (w <= h) 
      gluOrtho2D (0.0, 2.0, -0.5 * (GLfloat) h/(GLfloat) w, 
         1.5 * (GLfloat) h/(GLfloat) w);
   else 
      gluOrtho2D (0.0, 2.0*(GLfloat) w/(GLfloat) h, -0.5, 1.5); 
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
}

static void
glutscreen_display_list_input(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  glutscreen_t *this = (glutscreen_t *)o;
  display_list_t *input = (display_list_t *)fts_get_object(at);
  if (input && glIsList(input->list_index))
    currentlist = input->list_index;
}

static void
glutscreen_init(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{ 
  glutscreen_t *this = (glutscreen_t *)o;

  ac--;
  at++;
  
/*    if(fts_is_symbol(at)) */
/*      { */
/*        fts_symbol_t name = fts_get_symbol(at); */
/*        int i = 1;  // running argument index  */
/*        int size = 0; */
      
/*        if(fts_is_number(at + i)) */
/*  	{ */
/*  	  size = fts_get_number_int(at + i); */
/*  	  i++; */
/*  	} */

/*        if( size <= 0) */


/*      } */

  //create window 
  glutscreen_displaymode(o,0,0,0,0);
  glutscreen_window_position(o,0,0,0,0);
  glutscreen_window_size(o,0,0,0,0);
  glutscreen_window_title(o,0,0,0,0);

  glutscreen_background_color(o,0,0,0,0);
  glutscreen_shade_model(o,0,0,0,0);

  if (this->fullscreen) {
    glutSetCursor(GLUT_CURSOR_NONE);
    glutFullScreen();
  }

  glutDisplayFunc(display);
  glutReshapeFunc(reshape);
  // glutKeyboardFunc(keyboard);
  // glutSpecialFunc(arrows);
  glutIdleFunc(nothing);
  // glutMouseFunc(button);
  // glutPassiveMotionFunc(mousemove);

  glutMainLoop();
  // glutSwapBuffers();



}

static void
glutscreen_delete(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{ 

  glutscreen_t *this = (glutscreen_t *)o;
}

static fts_status_t
glutscreen_instantiate(fts_class_t *cl, int ac, const fts_atom_t *at)
{
  fts_class_init(cl, sizeof(glutscreen_t), 1, 0, 0);
  
  fts_method_define_varargs(cl, fts_SystemInlet, fts_s_init, glutscreen_init);
  fts_method_define_varargs(cl, fts_SystemInlet, fts_s_delete, glutscreen_delete);

  fts_method_define_varargs(cl, 0, display_list_symbol, glutscreen_display_list_input);
  
  return fts_Success;
}

void
glutscreen_config(void)
{
  fts_class_install(fts_new_symbol("glutscreen"), glutscreen_instantiate);
}
