/*
 * jMax
 * Copyright (C) 1994, 1995, 1998, 1999 by IRCAM-Centre Georges Pompidou, Paris, France.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * See file LICENSE for further informations on licensing terms.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Author : Paul-Mathieu Brossier <Paul.Brossier@ircam.fr>
 */

#include <X11/Xlib.h>
#include <GL/glx.h>
#include <fts/fts.h>

#include "../vectors/vectors.h"

#define EVENT_MASK ExposureMask | StructureNotifyMask

typedef struct _glxscreen_
{
  fts_object_t o;
  Display *x_display;
  XVisualInfo *x_visual;
  XSetWindowAttributes x_winattrib;
  Window x_window;
  GLXContext glx_context;
  GLint shinny;
  int light;
} glxscreen_t;

static int defSingle[]= {GLX_RGBA, GLX_RED_SIZE, 1, GLX_GREEN_SIZE, 1, GLX_BLUE_SIZE, 1, None };
static int defDouble[]= {GLX_RGBA, GLX_RED_SIZE, 1, GLX_GREEN_SIZE, 1, GLX_BLUE_SIZE, 1, GLX_DOUBLEBUFFER, None };

static GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
static GLfloat mat_shininess[] = { 50.0 };
static GLfloat light_intensity[] = { 0.8, 0.8, 0.8, 1.0 };
static GLfloat light_position[] = { 0.0, 0.0, 10.0, 1.0 };

static void
glx_init_gl(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  glxscreen_t *this = (glxscreen_t *)o;

  //GL first initialisation
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  //set background color
  glClearColor (0.0, 0.0, 0.0, 0.0);
  // Enable Texture Mapping
  glEnable(GL_TEXTURE_2D);				

  //light
  glShadeModel (GL_SMOOTH);

  //initialize viewing values
  glLoadIdentity();
  glMatrixMode(GL_PROJECTION);
  glOrtho(-1.0, 1.0, -1.0, 1.0, -10.0, 10.0);

 if(this->light == 1)
    {
      //      glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
      //      glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
      glMatrixMode(GL_MODELVIEW);
      glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, light_intensity);
      glLightfv(GL_LIGHT0, GL_POSITION, light_position);
      glEnable(GL_LIGHTING);
      glEnable(GL_LIGHT0);
      glEnable(GL_DEPTH_TEST);

      glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
      glEnable(GL_COLOR_MATERIAL);

    }
  else
    {
      glDisable(GL_LIGHTING);
      glDisable(GL_LIGHT0);
      glDisable(GL_DEPTH_TEST);

    }

  glXMakeCurrent(this->x_display, this->x_window, this->glx_context);
  glXSwapBuffers(this->x_display,glXGetCurrentDrawable());
}

static void
glx_light_on(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  glxscreen_t *this = (glxscreen_t *)o;
  this->light = 1;
  glx_init_gl(o,0,0,0,0);
}

static void
glx_light_off(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  glxscreen_t *this = (glxscreen_t *)o;
  this->light = 0;
  glx_init_gl(o,0,0,0,0);
}

static void
glx_light_switch(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  glxscreen_t *this = (glxscreen_t *)o;
  int state = fts_get_number_int(at);
  this->light = state;
  glx_init_gl(o,0,0,0,0);
}

static void
glxscreen_display_list_input(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  glxscreen_t *this = (glxscreen_t *)o;
  display_list_t *input = (display_list_t *)fts_get_object(at);

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
  glColor3f (1.0, 1.0, 1.0);

  if (input && glIsList(input->list_index))
    glCallList(input->list_index);

  glGetError();
  //update the screen
  //glFlush();
  glXSwapBuffers(this->x_display,glXGetCurrentDrawable());
  //  glXWaitGL();
  //  glXWaitX();
}  

static void
glxscreen_init(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{ 
  glxscreen_t *this = (glxscreen_t *)o;

  if ((this->x_display = XOpenDisplay(0)) == NULL) {
    fts_object_set_error(o, "Can't open display!\n");
    return;
  }

  if ((this->x_visual = glXChooseVisual(this->x_display, 
					DefaultScreen(this->x_display), 
					defDouble))== NULL) {
    //defSingle))== NULL) {

    fts_object_set_error(o, "Can't find suitable visual!\n");
    return;
  }
  this->glx_context = glXCreateContext(this->x_display, this->x_visual, 0, GL_TRUE);
  this->x_winattrib.colormap = XCreateColormap(this->x_display, 
					       RootWindow(this->x_display, this->x_visual->screen),
					       this->x_visual->visual, 
					       AllocNone);
  this->x_winattrib.border_pixel = 0;
  this->x_winattrib.event_mask = EVENT_MASK;
  this->x_window = XCreateWindow(this->x_display, RootWindow(this->x_display, this->x_visual->screen), 
				 0, 0, 600, 600, 0,
				 this->x_visual->depth, InputOutput,
				 this->x_visual->visual,
				 CWBorderPixel | CWColormap | CWEventMask,
				 &this->x_winattrib);
  XSelectInput(this->x_display, this->x_window, EVENT_MASK);   
  XSetStandardProperties(this->x_display, this->x_window, "gMax", "gMax", None, 0, 0, NULL);
  XMapWindow(this->x_display, this->x_window);
  glXMakeCurrent(this->x_display, this->x_window, this->glx_context);

  this->light = 0 ;
  glx_init_gl((fts_object_t *)o,0,0,0,0);
}

static void
glxscreen_delete(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{ 
  glxscreen_t *this = (glxscreen_t *)o;
  XDestroyWindow(this->x_display, this->x_window);
  glXDestroyContext(this->x_display, this->glx_context);
}

static fts_status_t
glxscreen_instantiate(fts_class_t *cl, int ac, const fts_atom_t *at)
{
  fts_symbol_t s;
  ac--;
  at++;

  fts_class_init(cl, sizeof(glxscreen_t), 2, 0, 0);
  fts_method_define_varargs(cl, fts_SystemInlet, fts_s_init, glxscreen_init);
  fts_method_define_varargs(cl, fts_SystemInlet, fts_s_delete, glxscreen_delete);
  fts_method_define_varargs(cl, 0, fts_s_bang, glx_init_gl);

  fts_method_define(cl, 1, fts_s_bang, glx_light_on, 0, 0);
  fts_method_define(cl, 1, fts_new_symbol("stop"), glx_light_off, 0, 0);
  fts_method_define(cl, 1, fts_s_int, glx_light_switch, 0 , 0);


  fts_method_define_varargs(cl, 0, display_list_symbol, glxscreen_display_list_input);
  return fts_Success;
}

void
glxscreen_config(void)
{
  fts_class_install(fts_new_symbol("screen"), glxscreen_instantiate);
}
