/*
 * jMax
 * Copyright (C) 1994, 1995, 1998, 1999 by IRCAM-Centre Georges Pompidou, Paris, France.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * See file LICENSE for further informations on licensing terms.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Based on Max/ISPW by Miller Puckette.
 *
 * Authors: Maurizio De Cecco, Francois Dechelle, Enzo Maggi, Norbert Schnell.
 *
 * Author : Paul-Mathieu Brossier <Paul.Brossier@ircam.fr>
 */

#include <X11/Xlib.h>
#include <GL/glx.h>

#include <fts/fts.h>

#include <../packages/data/c/src/ivec.h>
#include <../packages/data/c/src/fvec.h>

#include <../packages/video/c/src/vectors/vectors.h>

#define EVENT_MASK ExposureMask | StructureNotifyMask

typedef struct _glxscreen_
{
  fts_object_t o;
  Display *x_display;
  XVisualInfo *x_visual;
  XSetWindowAttributes x_winattrib;
  Window x_window;
  GLXContext glx_context;
  XEvent event;
  char *message;     /* Message printed in window */
} glxscreen_t;

static char progDefaultMessage[] = {"jMax OpenGL Graphics"};

static int defAttrib[]= {GLX_RGBA, GLX_RED_SIZE, 1, GLX_GREEN_SIZE, 1, GLX_BLUE_SIZE, 1, None };
static int defdouble[]= {GLX_RGBA, GLX_RED_SIZE, 1, GLX_GREEN_SIZE, 1, GLX_BLUE_SIZE, 1, GLX_DOUBLEBUFFER, None };

static int sglBuf24[] = {GLX_RGBA, GLX_RED_SIZE, 8, GLX_GREEN_SIZE, 8, GLX_BLUE_SIZE, 8, GLX_DEPTH_SIZE, 16, None};
static int dblBuf24[] = {GLX_RGBA, GLX_RED_SIZE, 4, GLX_GREEN_SIZE, 4, GLX_BLUE_SIZE, 4, GLX_DEPTH_SIZE, 16, GLX_DOUBLEBUFFER, None};
static int  sglBuf8[] = {GLX_RGBA, GLX_RED_SIZE, 3, GLX_GREEN_SIZE, 3, GLX_BLUE_SIZE, 2, GLX_DEPTH_SIZE, 16, None};
static int  dblBuf8[] = {GLX_RGBA, GLX_RED_SIZE, 1, GLX_GREEN_SIZE, 2, GLX_BLUE_SIZE, 1, GLX_DEPTH_SIZE, 16, GLX_DOUBLEBUFFER, None};

static Bool WaitForNotify(Display *d, XEvent *e, char *arg) {
    return (e->type == MapNotify) && (e->xmap.window == (Window)arg); }

void liteinit(void) 
{
   GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
   GLfloat mat_shininess[] = { 50.0 };
   GLfloat light_position[] = { 1.0, 1.0, 1.0, 0.0 };
   glClearColor (0.0, 0.0, 0.0, 0.0);
   glShadeModel (GL_SMOOTH);

   glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
   glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
   glLightfv(GL_LIGHT0, GL_POSITION, light_position);

   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);
   glEnable(GL_DEPTH_TEST);
}


static void
glxscreen_display_list_input(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  glxscreen_t *this = (glxscreen_t *)o;
  display_list_t *input = (display_list_t *)fts_get_object(at);

  //clear the screen
  //  glClear(GL_COLOR_BUFFER_BIT);

  //white is default drawing color
  glColor3f (1.0, 1.0, 1.0);

  //draw the input list
  if (input && glIsList(input->list_index))
    glCallList(input->list_index);

  //  glGetError();

  //update the screen
  //  glFlush();

  //  glXSwapBuffers(this->x_display,glXGetCurrentDrawable());
  glXWaitGL();
  glXWaitX();


}  

static void
glxscreen_init(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{ 
  glxscreen_t *this = (glxscreen_t *)o;
  int sAttribList;
  XEvent report;


  ac--;
  at++;

  //This part init a GLX window.
  //should work roughly on all X Window Systems

  //query the display
  if ((this->x_display = XOpenDisplay(0)) == NULL) {
    fts_object_set_error(o, "Can't open display!\n");
    return;
  }

  // Obtain the desired X Visual
  if(ac >= 1 && fts_is_number(at))
    {
      int n = fts_get_number_int(at);
      switch(n)
	{
	case 0: 
	  sAttribList = *defAttrib;
	  post("Glxscreen : Default mode set.\n");
	case 1: 
	  sAttribList = *sglBuf24;
	  post("Glxscreen : Single Buffer, 24 bits.\n");
	case 2: 
	  sAttribList =  *sglBuf8;
	  post("Glxscreen : Single Buffer,  8 bits.\n");
	case 3: 
	  sAttribList = *dblBuf24;
	  post("Glxscreen : Double Buffer, 24 bits.\n");
	case 4: 
	  sAttribList =  *dblBuf8;
	  post("Glxscreen : Double Buffer,  8 bits.\n");
	default: 
	  sAttribList =  *dblBuf8;
	  post("Glxscreen : Default mode set.\n");
	  break;
	}
    }
  else
    //    sAttribList = *defAttrib;
    sAttribList = *defdouble;

  if ((this->x_visual = glXChooseVisual(this->x_display, 
					DefaultScreen(this->x_display), 
					defAttrib))== NULL) {
    fts_object_set_error(o, "Can't find suitable visual!\n");
    return;
  }
  this->glx_context = glXCreateContext(this->x_display, this->x_visual, 0, GL_TRUE);
  
  // more X specific stuff
      
  // set x_winattrib
  this->x_winattrib.colormap = XCreateColormap(this->x_display, 
					       RootWindow(this->x_display, this->x_visual->screen),
					       this->x_visual->visual, 
					       AllocNone);
  this->x_winattrib.border_pixel = 0;

  //  this->x_winattrib.event_mask = StructureNotifyMask;
  this->x_winattrib.event_mask = EVENT_MASK;

  // create x_window and draw it
  this->x_window = XCreateWindow(this->x_display, RootWindow(this->x_display, this->x_visual->screen), 
				 0, 0, 600, 600, 0,
				 this->x_visual->depth, InputOutput,
				 this->x_visual->visual,
				 CWBorderPixel | CWColormap | CWEventMask,
				 &this->x_winattrib);

  XSelectInput(this->x_display, this->x_window, EVENT_MASK);   
  XSetStandardProperties(this->x_display, this->x_window, "jMaxVideo", "jMaxVideo", None, 0, 0, NULL);

  XMapWindow(this->x_display, this->x_window);

  // set window with GLX context
  glXMakeCurrent(this->x_display, this->x_window, this->glx_context);

  XIfEvent(this->x_display, &report, WaitForNotify, (char*)this->x_window);

  //GL first initialisation
  glClear(GL_COLOR_BUFFER_BIT);
  glFlush();

  //set background color
  glClearColor (0.0, 0.0, 0.0, 0.0);

  // Enable Texture Mapping
  glEnable(GL_TEXTURE_2D);				
  liteinit();
  //initialize viewing values
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);
  

  fts_object_refer(o);

}

static void
glxscreen_delete(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{ 
  glxscreen_t *this = (glxscreen_t *)o;

  //something is missing here

  if (this->x_display)
    {
      if (this->x_window)
      	XDestroyWindow(this->x_display, this->x_window);
      if (this->glx_context)
	glXDestroyContext(this->x_display, this->glx_context);
      if (this->x_winattrib.colormap)
	XFreeColormap(this->x_display , this->x_winattrib.colormap);
      XCloseDisplay(this->x_display);
    }

  //  this->x_display = NULL;
  //  this->x_window = NULL;
  //  this->x_winattrib = NULL;
  //  this->glx_context = NULL;
  
  fts_object_release(o);
}

static fts_status_t
glxscreen_instantiate(fts_class_t *cl, int ac, const fts_atom_t *at)
{
  fts_class_init(cl, sizeof(glxscreen_t), 1, 0, 0);
  
  fts_method_define_varargs(cl, fts_SystemInlet, fts_s_init, glxscreen_init);
  fts_method_define_varargs(cl, fts_SystemInlet, fts_s_delete, glxscreen_delete);
  
  fts_method_define_varargs(cl, 0, display_list_symbol, glxscreen_display_list_input);

  return fts_Success;
}

void
glxscreen_config(void)
{
  fts_class_install(fts_new_symbol("screen"), glxscreen_instantiate);
}
