/*
 * jMax
 * Copyright (C) 1994, 1995, 1998, 1999 by IRCAM-Centre Georges Pompidou, Paris, France.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * See file LICENSE for further informations on licensing terms.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Based on Max/ISPW by Miller Puckette.
 *
 * Authors: Maurizio De Cecco, Francois Dechelle, Enzo Maggi, Norbert Schnell.
 *
 * Author : Paul-Mathieu Brossier <Paul.Brossier@ircam.fr>
 */

#include <fts/fts.h>
#include "applytex.h"

void
gfx_bang(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  gfx_obj_t *this = (gfx_obj_t *)o;
  display_list_t *new_display_list;
  fts_atom_t a;
  int new_display_list_index = glGenLists(1);

  if (new_display_list_index != 0) { 
    glNewList(new_display_list_index, GL_COMPILE);     

    if(glIsTexture(this->argc))
      glBindTexture(GL_TEXTURE_2D, this->argc);

    if (this->display_list && glIsList(this->display_list->list_index))
      glCallList(this->display_list->list_index);

    glBindTexture(GL_TEXTURE_2D, 0);

    glEndList();
  }

  new_display_list = (display_list_t *)fts_object_create(display_list_class, 1, &a);

  new_display_list->list_index = new_display_list_index;

  fts_set_object_with_type(&a, new_display_list, display_list_symbol);

  fts_outlet_send(o, 0, display_list_symbol, 1, &a);

}

void
gfx_display_list_receive(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  gfx_obj_t *this = (gfx_obj_t *)o;

  if (this->display_list)
    fts_object_release((fts_object_t *)this->display_list);

  this->display_list = (display_list_t *)fts_get_object(at);

  fts_object_refer((fts_object_t *)this->display_list);

  gfx_bang(o, 0, 0, 0, 0);
}

void
gfx_params_receive(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  gfx_obj_t *this = (gfx_obj_t *)o;
  texture_t *input = (texture_t *)fts_get_object(at);
  //argc is here used to store the texture number
  this->argc = input->texture_index;
  post("argc : %d \n", this->argc);
}
  
static void
gfx_delete(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{ 
  gfx_obj_t *this = (gfx_obj_t *)o;

  if( this->display_list )
    fts_object_release((fts_object_t *)this->display_list);
}

static void
gfx_init(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  gfx_obj_t *this = (gfx_obj_t *)o;
  ac--;
  at++;

}

static fts_status_t
gfx_instantiate(fts_class_t *cl, int ac, const fts_atom_t *at)
{
  fts_class_init(cl, sizeof(gfx_obj_t), 2, 1, 0);
  fts_method_define_varargs(cl, fts_SystemInlet, fts_s_init, gfx_init);
  fts_method_define_varargs(cl, fts_SystemInlet, fts_s_delete, gfx_delete);

  /* inlet 0 methods */
  fts_method_define_varargs(cl, 0, fts_s_bang, gfx_bang);
  fts_method_define_varargs(cl, 0, display_list_symbol, gfx_display_list_receive);

  /* inlet 1 method */
  fts_method_define_varargs(cl, 1, texture_symbol, gfx_params_receive);

  return fts_Success;
}

void
gfx_applytex_config(void)
{
  fts_class_install(fts_new_symbol("applytex"), gfx_instantiate);
}
