/*
 * jMax
 * Copyright (C) 1994, 1995, 1998, 1999 by IRCAM-Centre Georges Pompidou, Paris, France.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * See file LICENSE for further informations on licensing terms.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Author : Paul-Mathieu Brossier <Paul.Brossier@ircam.fr>
 */

#include <fts/fts.h>

#include <stdio.h> 
#include <stdlib.h>

#include <GL/glx.h>

#include "pixmaps.h"

void gfx_picture_bmp_open(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at);
void gfx_picture_texture_receive(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at);
void gfx_picture_bang(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at);
void gfx_picture_gen(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at);


void
gfx_picture_bmp_open(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{ 
  pixmap_t *this = (pixmap_t *)o;
  FILE *file;
  unsigned long int size, i;
  unsigned short int planes, bpp;
  char temp;

  //if not found
  if ((file = fopen(fts_symbol_name(this->filename), "r"))==NULL)
    {
      fts_object_set_error(o, "File Not Found");
      post("File not found: %s\n",fts_symbol_name(this->filename));
    }
  else 
    {
      post("Bmp File %s opened\n", fts_symbol_name(this->filename));
      fseek(file, 18, SEEK_CUR);
      //seek width
      if ((i = fread(&this->sizeX, 4, 1, file)) != 1)
	post("Error reading width from %s.\n", fts_symbol_name(this->filename));
      else
	post("File Width : %d\n" , this->sizeX);
      //seek height
      if ((i = fread(&this->sizeY, 4, 1, file)) != 1) 
	post("Error reading width from %s.\n", fts_symbol_name(this->filename));
      else
	post("File Heigth : %d\n" , this->sizeX);
      // calculate the size (assuming 24 bits or 3 bytes per pixel).
      size = this->sizeX * this->sizeY * 3;
      post("Image size : %d\n", size);
      // read the planes
      if ((fread(&planes, 2, 1, file)) != 1)
	{
	  post("Error reading planes from %s.\n", fts_symbol_name(this->filename));
	  return;
	}
      if (planes != 1)
	{
	  post("Planes from %s are not 1: %d\n", fts_symbol_name(this->filename), planes);
	  return;
	}
      // read the bpp 
      if ((i = fread(&bpp, 2, 1, file)) != 1) 
	{
	  post("Error reading bpp from %s.\n", fts_symbol_name(this->filename));
	  return;
	}
      if (bpp != 24)
	{
	  post("Bpp from %s is not 24: %d\n",fts_symbol_name(this->filename), bpp); 
	      return;
	}
      // seek past the rest of the bitmap header. 
      fseek(file, 24, SEEK_CUR); 
      // read the data.  
      this->data = (char *) fts_malloc (size); 
      if (this->data == NULL) 
	{  
	  post("Error allocating memory for image data\n"); 
	  return; 
	} 
      if ((i = fread(this->data, size, 1, file)) != 1)  
	{ 
	  post("Error reading image data from %s.\n", fts_symbol_name(this->filename)); 	       
	  return; 
	} 
      // reverse all of the colors. (bgr -> rgb)  
      for (i=0;i<size;i+=3)  
	{ 
	  temp = this->data[i]; 
	  this->data[i] = this->data[i+2];  
	  this->data[i+2]= temp; 
	}
      // Bug here when no screen.
      //need to check before a call
    }
}  

void
gfx_picture_gen(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  pixmap_t *this = (pixmap_t *)o;

  if(glIsTexture(this->texture_index))
    glDeleteTextures(1, &this->texture_index);

  glGenTextures(1, &this->texture_index); 

  // 2d texture (x and y size) 
  glBindTexture(GL_TEXTURE_2D, this->texture_index);   
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); // scale linearly when image bigger than texture 
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); // scale linearly when image smalled than texture 
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  // 2d texture, level of detail 0 (normal), 3 components (red, green, blue),
  // x size from image, y size from image, border 0 (normal), rgb color data, 
  // unsigned byte data, and finally the data itself.
  glTexImage2D(GL_TEXTURE_2D, 0, 3, this->sizeX, this->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, this->data); 

  this->data = 0;
  fts_free(this->data);
  
  //generate auto tex coords.
  glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
  glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
  glEnable(GL_TEXTURE_GEN_S);
  glEnable(GL_TEXTURE_GEN_T);

  glBindTexture(GL_TEXTURE_2D, 0);

  post("Texture %d generated from %s.\n",this->texture_index,  fts_symbol_name(this->filename));
}

void
gfx_picture_bang(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  pixmap_t *this = (pixmap_t *)o;
  texture_t *new_texture;
  fts_atom_t a;
  gfx_picture_bmp_open(o,0,0,0,0);
  gfx_picture_gen(o,0,0,0,0);
  new_texture = (texture_t *)fts_object_create(texture_class, 1, &a);
  new_texture->texture_index = this->texture_index;
  fts_set_object_with_type(&a, new_texture, texture_symbol);
  fts_outlet_send(o, 0, texture_symbol, 1, &a);
}

static void
gfx_picture_init(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  pixmap_t *this = (pixmap_t *)o;
  ac--;
  at++;
  this->filename = fts_get_symbol(at); 
  gfx_picture_bmp_open(o,0,0,0,0);
}

static void
gfx_picture_delete(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{ 
  pixmap_t *this = (pixmap_t *)o;
  //  fts_object_release(this->o);
}

static fts_status_t
gfx_picture_instantiate(fts_class_t *cl, int ac, const fts_atom_t *at)
{

  if(ac == 2 && fts_is_symbol(at+1))
    {  
      fts_class_init(cl, sizeof(pixmap_t), 1, 1, 0);
      fts_method_define_varargs(cl, fts_SystemInlet, fts_s_init, gfx_picture_init);
      fts_method_define_varargs(cl, fts_SystemInlet, fts_s_delete, gfx_picture_delete);
      fts_method_define_varargs(cl, 0, fts_s_bang, gfx_picture_bang);
    }
  else 
   return &fts_CannotInstantiate;
  
  return fts_Success;

}

void
gfx_picture_config(void)
{
  fts_class_install(fts_new_symbol("picture"), gfx_picture_instantiate);
}
