/*
 * jMax
 * Copyright (C) 1994, 1995, 1998, 1999 by IRCAM-Centre Georges Pompidou, Paris, France.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * See file LICENSE for further informations on licensing terms.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Based on Max/ISPW by Miller Puckette.
 *
 * Authors: Maurizio De Cecco, Francois Dechelle, Enzo Maggi, Norbert Schnell.
 *
 * Author : Paul-Mathieu Brossier <Paul.Brossier@ircam.fr>
 */

#include <fts/fts.h>

#include <GL/gl.h>

#include <../packages/data/c/src/ivec.h>
#include <../packages/data/c/src/fvec.h>

#include "vectors.h"
#include "drawvects.h"

static void
drawvects_bang(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  drawvects_t *this = (drawvects_t *)o; 
  display_list_t *new_display_list;
  fts_atom_t a;
  int i = 0;

  if(glIsList(this->new_display_list_index))
    glDeleteLists(this->new_display_list_index,1);
  this->new_display_list_index = glGenLists(1);
  post("debugging: list %d generated for drawvects with mode %d\n", this->new_display_list_index, this->mode);
  
  if (this->new_display_list_index != 0) { 
    glNewList(this->new_display_list_index, GL_COMPILE);     
    
    switch(this->mode)
      {
      case 0:
	glBegin(GL_POINTS);
      case 1:
	glBegin(GL_LINE_STRIP);
      case 2:
	glBegin(GL_LINE_LOOP);
      case 3:
	glBegin(GL_TRIANGLES);
      case 4:
	glBegin(GL_TRIANGLE_STRIP);
      case 5:
	glBegin(GL_TRIANGLE_FAN);
      case 6:
	glBegin(GL_QUADS);
      case 7:
	glBegin(GL_QUAD_STRIP);
      case 8:
	glBegin(GL_POLYGON);
	//      default :
	//	glBegin(GL_TRIANGLES);
      }
    while(   (i < fvec_get_size(this->Xref.fvec))
	     && (i < fvec_get_size(this->Yref.fvec)) 
	     && (i < fvec_get_size(this->Zref.fvec)) 
	     && (3 * i < fvec_get_size(this->Cref.fvec)))
      {
	glColor3f (fvec_get_element(this->Cref.fvec, i),
		   fvec_get_element(this->Cref.fvec, i+1),
		   fvec_get_element(this->Cref.fvec, i+2));
	glVertex3f(fvec_get_element(this->Xref.fvec, i),
		   fvec_get_element(this->Yref.fvec, i),
		   fvec_get_element(this->Zref.fvec, i));
	i++;
      } 
    glEnd();
    
    glEndList();
    
  }
  new_display_list = (display_list_t *)fts_object_create(display_list_class, 1, &a);
  
  new_display_list->list_index = this->new_display_list_index;
  
  fts_set_object_with_type( &a, new_display_list, display_list_symbol);
  
  fts_outlet_send(o, 0, display_list_symbol, 1, &a);
}

static void
drawvects_fvec_Xinput(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  drawvects_t *this = (drawvects_t *)o;
  this->Xref.fvec = fvec_atom_get(at);
}

static void
drawvects_ivec_Xinput(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  drawvects_t *this = (drawvects_t *)o;
  this->Xref.ivec = ivec_atom_get(at);
}

static void
drawvects_fvec_Yinput(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  drawvects_t *this = (drawvects_t *)o;
  this->Yref.fvec = fvec_atom_get(at);
}

static void
drawvects_ivec_Yinput(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  drawvects_t *this = (drawvects_t *)o;
  this->Yref.ivec = ivec_atom_get(at);
}

static void
drawvects_fvec_Zinput(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  drawvects_t *this = (drawvects_t *)o;
  this->Zref.fvec = fvec_atom_get(at);
}

static void
drawvects_ivec_Zinput(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  drawvects_t *this = (drawvects_t *)o;
  this->Zref.ivec = ivec_atom_get(at);
}

static void
drawvects_fvec_Cinput(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  drawvects_t *this = (drawvects_t *)o;
  this->Cref.fvec = fvec_atom_get(at);
}

static void
drawvects_ivec_Cinput(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  drawvects_t *this = (drawvects_t *)o;
  this->Cref.ivec = ivec_atom_get(at);
}

static void
drawvects_init(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  drawvects_t *this = (drawvects_t *)o;
  ac--;
  at++;

  this->mode = 0;
  this->type = 0;
  this->dims = 0;

  if(ac >= 2 && fts_is_number( at + 2 )) 
    this->dims  = fts_get_number_int( at + 2 );

  if(ac >= 1 && fts_is_number( at + 1 ))
    this->type  = fts_get_number_int( at + 1 );

  if(ac >= 0 && fts_is_number( at ))
    this->mode  = fts_get_number_int( at );

  post("Mode : %d, Type: %d, Dims: %d\n", this->mode, this->type, this->dims);

/*    fts_object_refer(this->ref.obj); */
}

static void
drawvects_delete(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{ 
  drawvects_t *this = (drawvects_t *)o;
/*    fts_object_release(this->ref.obj); */
}

static fts_status_t
drawvects_instantiate(fts_class_t *cl, int ac, const fts_atom_t *at)
{
  fts_class_init(cl, sizeof(drawvects_t), 4, 1, 0);
  fts_method_define_varargs(cl, fts_SystemInlet, fts_s_init, drawvects_init);
  fts_method_define_varargs(cl, fts_SystemInlet, fts_s_delete, drawvects_delete);

  //first inlet : X coords
  fts_method_define_varargs(cl, 0, fvec_symbol, drawvects_fvec_Xinput); 
  fts_method_define_varargs(cl, 0, ivec_symbol, drawvects_ivec_Xinput);
  fts_method_define_varargs(cl, 1, fvec_symbol, drawvects_fvec_Yinput);
  fts_method_define_varargs(cl, 1, ivec_symbol, drawvects_ivec_Yinput);
  fts_method_define_varargs(cl, 2, fvec_symbol, drawvects_fvec_Zinput);
  fts_method_define_varargs(cl, 2, ivec_symbol, drawvects_ivec_Zinput);
  fts_method_define_varargs(cl, 3, fvec_symbol, drawvects_fvec_Cinput);
  fts_method_define_varargs(cl, 3, ivec_symbol, drawvects_ivec_Cinput);


  fts_method_define_varargs(cl, 0, fts_s_bang, drawvects_bang);

  return fts_Success;
}

void
drawvects_config(void)
{
  fts_class_install(fts_new_symbol("drawvects"), drawvects_instantiate);
}

