/*
 * Jmax
 * Copyright (C) 1994, 1995, 1998, 1999 by IRCAM-Centre Georges Pompidou, Paris, France.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * See file LICENSE for further informations on licensing terms.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Based on Max/ISPW by Miller Puckette.
 *
 * Authors: Francois Dechelle, Norbert Schnell
 *
 * Author: Paul Brossier
 */

/* This code is simply an adapted cut.c version. */

#include <fts/fts.h>
#include "vectors.h"
#include "../../../../data/c/src/fvec.h"

fts_symbol_t gfxhistory_symbol = 0;

typedef struct _gfxhistory_ftl_
{
  fts_object_t o;
  fvec_t *fvec;
  int index;
  fts_alarm_t alarm;
} gfxhistory_ftl_t;

typedef struct _gfxhistory_
{
  fts_object_t o;
  float *histo[50];
  ftl_data_t x_data;
  int new_display_list_index;
  int age;
} gfxhistory_t;

/***************************************************************************************
 *
 *  user methods
 *
 */

static void gfxhistory_output(fts_alarm_t *alarm, void *o)
{
  gfxhistory_t *this = (gfxhistory_t *)o;
  //  gfxhistory_ftl_t *data = (gfxhistory_ftl_t *)ftl_data_get_ptr(this->x_data);
  //  fts_atom_t a[1];

  fts_alarm_unarm(alarm);

  //  fvec_atom_set(a, data->fvec);
  //  fts_outlet_send((fts_object_t *)o, 0, fvec_symbol, 1, a);
}

static void 
gfxhistory_bang(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  gfxhistory_t *this = (gfxhistory_t *)o;
  gfxhistory_ftl_t *x_data = (gfxhistory_ftl_t *)ftl_data_get_ptr(this->x_data);
  display_list_t *new_display_list;
  fts_atom_t b;
  int i,j;

  x_data->index = 0;

  if(glIsList(this->new_display_list_index))
    glDeleteLists(this->new_display_list_index,1);
  this->new_display_list_index = glGenLists(1);

#ifdef DEBUG
  post("New List: %d\n",this->new_display_list_index);
#endif

  if (this->new_display_list_index != 0) { 
    glNewList(this->new_display_list_index, GL_COMPILE);     
    for ( j=0; j <  fvec_get_size(y_data->fvec)-1; j++) {
      glBegin( GL_QUAD_STRIP );
       for ( i=0; i < fvec_get_size(x_data->fvec)-1; i++) 
	 { 
	   glVertex3f((float)0.01*i,  (float)0.01*j,  
  		      fvec_get_element(x_data->fvec,i) 
		      * fvec_get_element(y_data->fvec,j) );
	   glVertex3f((float)0.01*i, (float)0.01*(j+1), 
		      fvec_get_element(x_data->fvec,i+1) 
		      * fvec_get_element(y_data->fvec,j) );
	 }
       glEnd();
    }
    glEndList();
  }

  new_display_list = (display_list_t *)fts_object_create(display_list_class, 1, &b);

  new_display_list->list_index = this->new_display_list_index;

  fts_set_object_with_type(&b, new_display_list, display_list_symbol);

  fts_outlet_send((fts_object_t *)o, 0, display_list_symbol, 1, &b);
}

static void 
gfxhistory_set_x_fvec(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  gfxhistory_t *this = (gfxhistory_t *)o;
  gfxhistory_ftl_t *data = (gfxhistory_ftl_t *)ftl_data_get_ptr(this->x_data);
  fvec_t *fvec = fvec_atom_get(at);

  fts_object_release((fts_object_t *)data->fvec);
  data->fvec = fvec;
  fts_object_refer((fts_object_t *)fvec);

  data->index = fvec_get_size(fvec);
}

/***************************************************************************************
 *
 *  put
 *
 */

static void 
gfxhistory_put(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  gfxhistory_t *this = (gfxhistory_t *)o;
  fts_dsp_descr_t *dsp = (fts_dsp_descr_t *)fts_get_ptr(at);
  float x_sr = fts_dsp_get_input_srate(dsp, 0);
  int x_n_tick = fts_dsp_get_input_size(dsp, 0);
  fts_atom_t a[this->age];

  fts_set_ftl_data(a + 0, this->x_data);
  fts_set_symbol(a + 1, fts_dsp_get_input_name(dsp, 0));
  fts_set_int(a + 2, x_n_tick);
  fts_set_ftl_data(a + 3, this->y_data);
  fts_set_symbol(a + 4, fts_dsp_get_input_name(dsp, 1));
  fts_set_int(a + 5, y_n_tick);
  fts_dsp_add_function(gfxhistory_symbol, 6, a);
}

static void
gfxhistory_ftl(fts_word_t *argv)
{
  gfxhistory_ftl_t *x_data = (gfxhistory_ftl_t *)fts_word_get_ptr(argv + 0);
  float * restrict x_in = (float *) fts_word_get_ptr(argv + 1);
  int x_n_tick = fts_word_get_int(argv + 2);

  //
  float *x_buf = fvec_get_ptr(x_data->fvec);
  int x_size = fvec_get_size(x_data->fvec);
  int x_index = x_data->index;
  int x_n = x_size - x_index;
  //

  int i;

  if(x_n > x_n_tick)
    x_n = x_n_tick;
  else if(x_n)
    {
      fts_alarm_set_delay(&(x_data->alarm), 0.01);
      fts_alarm_arm(&(x_data->alarm));
    }

  for(i=0; i<x_n; i++)
    x_buf[x_index + i] = x_in[i];

  x_data->index += x_n;
}

/***************************************************************************************
 *
 *  class
 *
 */

static void
gfxhistory_init(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{ 
  gfxhistory_t *this = (gfxhistory_t *)o;
  gfxhistory_ftl_t *x_data,*y_data;
  fvec_t *x_fvec, *y_fvec;

  ac--;
  at++;

  this->x_data = ftl_data_alloc(sizeof(gfxhistory_ftl_t));
  x_data = ftl_data_get_ptr(this->x_data);
  
  if(fts_is_int(at))
    x_data->fvec = (fvec_t *)fts_object_create(fvec_class, 1, at);
  else
    x_data->fvec = fvec_atom_get(at);

  fts_object_refer((fts_object_t *)x_data->fvec);

  //  fts_object_refer((fts_object_t *)this->display_list);

  x_data->index = fvec_get_size(x_data->fvec);

  fts_alarm_init(&(x_data->alarm), 0, gfxhistory_output, (void *)this);

  fts_dsp_add_object((fts_object_t *)this);
}

static void
gfxhistory_delete(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  gfxhistory_t *this = (gfxhistory_t *)o;
  gfxhistory_ftl_t *x_data = ftl_data_get_ptr(this->x_data);
  gfxhistory_ftl_t *y_data = ftl_data_get_ptr(this->y_data);

  if(x_data->fvec)
    fts_object_release((fts_object_t *)x_data->fvec);    

  ftl_data_free(this->x_data);

  fts_dsp_remove_object(o);
  //  if( this->display_list )
  //    fts_object_release((fts_object_t *)this->display_list);
}

static fts_status_t
gfxhistory_instantiate(fts_class_t *cl, int ac, const fts_atom_t *at)
{  
  ac--;
  at++;

  if(ac == 2 && fts_is_int(at) && fts_is_int(at+1) )
    {
      fts_class_init(cl, sizeof(gfxhistory_t), 1, 1, 0);

      fts_method_define_varargs(cl, fts_SystemInlet, fts_s_init, gfxhistory_init);
      fts_method_define_varargs(cl, fts_SystemInlet, fts_s_delete, gfxhistory_delete);      
      fts_method_define_varargs(cl, fts_SystemInlet, fts_new_symbol("put"), gfxhistory_put);

      fts_method_define_varargs(cl, 0, fts_s_bang, gfxhistory_bang);
    }
  else
    return &fts_CannotInstantiate;

  dsp_sig_inlet(cl, 0);
  return fts_Success;
}

void
gfxhistory_config(void)
{
  gfxhistory_symbol = fts_new_symbol("gfxhistory~");
  dsp_declare_function(gfxhistory_symbol, gfxhistory_ftl);  

  fts_metaclass_install(gfxhistory_symbol, gfxhistory_instantiate, fts_arg_type_equiv);
}
