/*
 * jMax
 * Copyright (C) 1994, 1995, 1998, 1999 by IRCAM-Centre Georges Pompidou, Paris, France.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * See file LICENSE for further informations on licensing terms.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Based on Max/ISPW by Miller Puckette.
 *
 * Authors: Maurizio De Cecco, Francois Dechelle, Enzo Maggi, Norbert Schnell.
 *
 * Author : Paul-Mathieu Brossier <Paul.Brossier@ircam.fr>
 */

#include <fts/fts.h>
#include "template.h"

static void gfx_draw(fts_object_t *o);

void
gfx_bang(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  gfx_obj_t *this = (gfx_obj_t *)o;
  display_list_t *new_display_list;
  fts_atom_t a;

  if(glIsList(this->new_display_list_index))
    glDeleteLists(this->new_display_list_index,1);
  this->new_display_list_index = glGenLists(1);

#ifdef DEBUG
  post("New List: %d\n",this->new_display_list_index);
#endif

  if (this->new_display_list_index != 0) { 
    glNewList(this->new_display_list_index, GL_COMPILE);     
    gfx_draw(o);
    glEndList();
  }

  new_display_list = (display_list_t *)fts_object_create(display_list_class, 1, &a);
  new_display_list->list_index = this->new_display_list_index;
  fts_set_object_with_type(&a, new_display_list, display_list_symbol);
  fts_outlet_send(o, 0, display_list_symbol, 1, &a);
}

void
gfx_params_receive(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  gfx_obj_t *this = (gfx_obj_t *)o;
  int i;

  if(ac > GFX_NB_PARAMS)
    ac = GFX_NB_PARAMS;

  if(fts_is_number(at))
    this->argv[winlet-1] = at[0];
}
  
static void
gfx_delete(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{ 
  gfx_obj_t *this = (gfx_obj_t *)o;

  if( this->display_list )
    fts_object_release((fts_object_t *)this->display_list);
}

static void
gfx_init(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  gfx_obj_t *this = (gfx_obj_t *)o;
  int i;
  ac--;
  at++;
  for(i=0; i < GFX_NB_PARAMS; i++)
    {
      fts_set_float(this->argv + i, GFX_DEF_PARAMS[i]);
      gfx_params_receive(o, i+1, s, 1, at+i);
    }
}

static fts_status_t
gfx_instantiate(fts_class_t *cl, int ac, const fts_atom_t *at)
{
  int i;
  fts_class_init(cl, sizeof(gfx_obj_t), GFX_NB_PARAMS+1, 1, 0);
  fts_method_define_varargs(cl, fts_SystemInlet, fts_s_init, gfx_init);
  fts_method_define_varargs(cl, fts_SystemInlet, fts_s_delete, gfx_delete);

  /* inlet 0 methods */
  fts_method_define_varargs(cl, 0, fts_s_bang, gfx_bang);

  /* inlet 1 method */
  for(i=0; i < GFX_NB_PARAMS; i++)
    fts_method_define_varargs(cl, 1+i, fts_s_float, gfx_params_receive);

  return fts_Success;
}
