/*
 * Jmax
 * Copyright (C) 1994, 1995, 1998, 1999 by IRCAM-Centre Georges Pompidou, Paris, France.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * See file LICENSE for further informations on licensing terms.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Based on Max/ISPW by Miller Puckette.
 *
 * Authors: Maurizio De Cecco, Francois Dechelle, Enzo Maggi, Norbert Schnell.
 *
 * Author : Paul-Mathieu Brossier <Paul.Brossier@ircam.fr>
 *
 * Thanx to llardin@dsimprove.be
 */

#include <stdlib.h>

static int GFX_NB_PARAMS = 6;
static float GFX_DEF_PARAMS[6] = { 45., 1., 0.2, 50., 9.81, 0.01};
//angle speed spread partnumber gravity part_size
#include "template.c"

struct gfx_particle
{
  float t;       /* Live time */
  float vx,vy,vz;       /* Speed */
  struct gfx_particle *next, *prev;
};

int i;
float angle, speed, spread, ttl, gravity, part_size;
struct gfx_particle *fn; /* Fountain */

void gfx_add_particles(void)
{
  struct gfx_particle *part_temp;
  int i;

  for (i=0; i<2; i++)
  {
    part_temp = (struct gfx_particle *)fts_malloc(sizeof(struct gfx_particle));
    if (fn) fn->prev = part_temp;
    part_temp->next = fn;
    fn = part_temp;
  
    part_temp->t = 0;

    part_temp->vx = (float)(rand() % 10000)/40000*spread+speed*cos(6.2832*angle/360.);
    part_temp->vy = (float)(rand() % 10000)/40000*spread+speed*sin(6.2832*angle/360.);
    part_temp->vz = (float)(rand() % 10000)/40000*spread;
    //    part_temp->v =  (float)(rand() % 10000)/4000+10;
  }
}

void
gfx_draw(fts_object_t *o)
{
  gfx_obj_t *this = (gfx_obj_t *)o;
  float dt, x, y, z;

  struct gfx_particle *part_temp1, *part_temp2;

  angle =  fts_get_number_float(this->argv + 0) ;
  speed =  fts_get_number_float(this->argv + 1) ;  
  spread = fts_get_number_float(this->argv + 2) ;
  ttl =    fts_get_number_float(this->argv + 3) / 10 ;
  gravity= fts_get_number_float(this->argv + 4) ;
  part_size = fts_get_number_float(this->argv + 5) ;
  dt=0.1;

  if (ttl == 0.) ttl=1.;

  gfx_add_particles();
  part_temp1 = fn;

  glPushMatrix();

  glBegin(GL_QUADS);
   while (part_temp1)
     {

       x = part_temp1->vx * part_temp1->t; 
       y = (part_temp1->t * part_temp1->vy
	    - gravity * part_temp1->t * part_temp1->t / 20 );
       z = part_temp1->vz * part_temp1->t ;

       //#ifdef DEBUG       
       post("x,y,z : %e, %e, %e \n", x,y,z);
       post("vx,vy,vz : %e, %e, %e \n", part_temp1->vx,part_temp1->vy,part_temp1->vz);
       post("t : %e \n", part_temp1->t);
       //#endif
       glPushMatrix();
       glTranslated(x,y,z);
       //   gluSphere(pQuadric,part_size,12,12);        
       if (this->display_list && glIsList(this->display_list->list_index))
	 glCallList(this->display_list->list_index);
       post("%i\n",this->display_list->list_index);
       glPopMatrix();

       if (part_temp1->t < ttl) 
	 part_temp1->t += dt;
       else
	 {
	   // delete this particle
	   part_temp2 = part_temp1->prev;
	   part_temp1->prev->next = part_temp1->next;
	   if (part_temp1->next) part_temp1->next->prev = part_temp2;
	   fts_free(part_temp1);
	   part_temp1 = part_temp2;
	 }
       part_temp1 = part_temp1->next;
     }
  glEnd();
  
  glPopMatrix();
}

void
gfx_fountain2_config(void)
{
  fts_class_install(fts_new_symbol("gfountain2"), gfx_instantiate);
}
