/*
 * jMax
 * Copyright (C) 1994, 1995, 1998, 1999 by IRCAM-Centre Georges Pompidou, Paris, France.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * See file LICENSE for further informations on licensing terms.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Author : Paul-Mathieu Brossier <Paul.Brossier@ircam.fr>
 */

#include <fts/fts.h>
#include "vectors_fx.h"

typedef struct _gfxmultiple_obj_
{
  fts_object_t o;
  int new_display_list_index;
  display_list_t *display_list_right; //display_lists received
  display_list_t *display_list_left; //display_lists received
} gfxmultiple_obj_t;

void
gfxmultiple_bang(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  gfxmultiple_obj_t *this = (gfxmultiple_obj_t *)o;
  display_list_t *new_display_list;
  fts_atom_t a;
  int i;

  if(glIsList(this->new_display_list_index))
    glDeleteLists(this->new_display_list_index,1);
  this->new_display_list_index = glGenLists(1);

#ifdef DEBUG
  post("New effect List: %d\n",this->new_display_list_index);
#endif

  if (this->new_display_list_index != 0) { 
    glNewList(this->new_display_list_index, GL_COMPILE);     
    if (this->display_list_left && glIsList(this->display_list_left->list_index))
      glCallList(this->display_list_left->list_index);
    if (this->display_list_right && glIsList(this->display_list_right->list_index))
      glCallList(this->display_list_right->list_index);
    glEndList();
  }
  new_display_list = (display_list_t *)fts_object_create(display_list_class, 1, &a);
  new_display_list->list_index = this->new_display_list_index;
  fts_set_object_with_type(&a, new_display_list, display_list_symbol);
  fts_outlet_send(o, 0, display_list_symbol, 1, &a);
}

void
gfxmultiple_display_list_receive(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  gfxmultiple_obj_t *this = (gfxmultiple_obj_t *)o;
  if (this->display_list_left)
    fts_object_release((fts_object_t *)this->display_list_left);
  this->display_list_left = (display_list_t *)fts_get_object(at);
  fts_object_refer((fts_object_t *)this->display_list_left);
  gfxmultiple_bang(o, 0, 0, 0, 0);
}

void
gfxmultiple_list_receive(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  gfxmultiple_obj_t *this = (gfxmultiple_obj_t *)o;
  if (this->display_list_right)
    fts_object_release((fts_object_t *)this->display_list_right);
  this->display_list_right = (display_list_t *)fts_get_object(at);
  fts_object_refer((fts_object_t *)this->display_list_right);
}
  
static void
gfxmultiple_delete(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{ 
  gfxmultiple_obj_t *this = (gfxmultiple_obj_t *)o;
  int i;
  if( this->display_list_right )
    fts_object_release((fts_object_t *)this->display_list_right);
  if( this->display_list_left )
    fts_object_release((fts_object_t *)this->display_list_left);
}

static void
gfxmultiple_init(fts_object_t *o, int winlet, fts_symbol_t s, int ac, const fts_atom_t *at)
{
  gfxmultiple_obj_t *this = (gfxmultiple_obj_t *)o;
  ac--;
  at++;
}

static fts_status_t
gfxmultiple_instantiate(fts_class_t *cl, int ac, const fts_atom_t *at)
{
  fts_class_init(cl, sizeof(gfxmultiple_obj_t), 2, 1, 0);
  fts_method_define_varargs(cl, fts_SystemInlet, fts_s_init, gfxmultiple_init);
  fts_method_define_varargs(cl, fts_SystemInlet, fts_s_delete, gfxmultiple_delete);
  
  /* inlet 0 methods */
  fts_method_define_varargs(cl, 0, fts_s_bang, gfxmultiple_bang);
  fts_method_define_varargs(cl, 0, display_list_symbol, gfxmultiple_display_list_receive);
  
  /* other inlets methods */
  fts_method_define_varargs(cl, 1, display_list_symbol, gfxmultiple_list_receive);
  return fts_Success;
}

void
gfxmultiple_config(void)
{
  fts_class_install(fts_new_symbol("gfxpack"), gfxmultiple_instantiate);
}
